﻿using System;
using System.Runtime.Serialization;
using Microwave.Classes.Interfaces;

namespace Microwave.Classes.Controllers
{
    public class UserInterface : IUserInterface
    {
        private enum States
        {
            READY, SETPOWER, SETTIME, SETDEFROST, COOKING, DOOROPEN
        }

        private States myState = States.READY;

        private ICookController myCooker;
        private ILight myLight;
        private IDisplay myDisplay;
        private IBuzzer myBuzzer;

        private int powerLevel = 50;
        private int time = 1;

        public UserInterface(
            IButton powerButton,
            IButton timeButton,
            IButton startCancelButton,
            IButton defrostButton,
            IDoor door,
            IDisplay display,
            ILight light,
            IBuzzer buzzer,
            ICookController cooker)
        {
            powerButton.Pressed += new EventHandler(OnPowerPressed);
            timeButton.Pressed += new EventHandler(OnTimePressed);
            startCancelButton.Pressed += new EventHandler(OnStartCancelPressed);
            defrostButton.Pressed += new EventHandler(OnDefrostPressed);

            door.Closed += new EventHandler(OnDoorClosed);
            door.Opened += new EventHandler(OnDoorOpened);

            myCooker = cooker;
            myLight = light;
            myDisplay = display;
            myBuzzer = buzzer;
        }

        private void ResetValues()
        {
            powerLevel = 50;
            time = 1;
        }

        public void OnPowerPressed(object sender, EventArgs e)
        {
            switch (myState)
            {
                case States.READY:
                    myDisplay.ShowPower(powerLevel);
                    myState = States.SETPOWER;
                    break;
                case States.SETPOWER:
                    powerLevel = (powerLevel >= myCooker.MaxPower ? 50 : powerLevel+50);
                    myBuzzer.Buzz(1, 200, "keybeep.wav");
                    myDisplay.ShowPower(powerLevel);
                    break;
            }
        }

        public void OnTimePressed(object sender, EventArgs e)
        {
            switch (myState)
            {
                case States.SETPOWER:
                    myDisplay.ShowTime(time, 0);
                    myState = States.SETTIME;
                    break;
                case States.SETTIME:
                    time += 1;
                    myBuzzer.Buzz(1, 200, "keybeep.wav");
                    myDisplay.ShowTime(time, 0);
                    break;
                case States.COOKING:
                    myCooker.AddCookingTime();
                    break;
            }
        }

        public void OnStartCancelPressed(object sender, EventArgs e)
        {
            switch (myState)
            {
                case States.SETPOWER:
                    ResetValues();
                    myDisplay.Clear();
                    myState = States.READY;
                    break;
                case States.SETTIME:
                    myLight.TurnOn();
                    myBuzzer.Buzz(1, 1000, "yes.wav");
                    myCooker.StartCooking(powerLevel, time*60);
                    myState = States.COOKING;
                    break;
                case States.COOKING:
                    ResetValues();
                    myCooker.Stop();
                    myLight.TurnOff();
                    myDisplay.Clear();
                    myBuzzer.Buzz(1, 1200, "ohoh.wav");
                    myState = States.READY;
                    break;
                case States.SETDEFROST:
                    myLight.TurnOn();
                    myCooker.StartCooking(powerLevel, time * 60);
                    myCooker.Defrost(powerLevel);
                    myBuzzer.Buzz(1, 1200, "ok.wav");
                    myState = States.COOKING;
                    break;
            }
        }

        public void OnDefrostPressed(object sender, EventArgs e)
        {
            switch (myState)
            {
                case States.SETTIME:
                    myState = States.SETDEFROST;
                    break;
                case States.SETPOWER:
                    myState = States.SETDEFROST;
                    break;
            }
        }

        public void OnDoorOpened(object sender, EventArgs e)
        {
            switch (myState)
            {
                case States.READY:
                    myLight.TurnOn();
                    myState = States.DOOROPEN;
                    break;
                case States.SETPOWER:
                    ResetValues();
                    myLight.TurnOn();
                    myDisplay.Clear();
                    myState = States.DOOROPEN;
                    break;
                case States.SETTIME:
                    ResetValues();
                    myLight.TurnOn();
                    myDisplay.Clear();
                    myState = States.DOOROPEN;
                    break;
                case States.COOKING:
                    myCooker.Stop();
                    myDisplay.Clear();
                    myBuzzer.Buzz(1, 1000, "ohoh.wav");
                    ResetValues();
                    myState = States.DOOROPEN;
                    break;
            }
        }

        public void OnDoorClosed(object sender, EventArgs e)
        {
            switch (myState)
            {
                case States.DOOROPEN:
                    myLight.TurnOff();
                    myDisplay.CookingDone();
                    myBuzzer.Buzz(1, 1000, "ok.wav");
                    myState = States.READY;
                    break;
            }
        }

        public void CookingIsDone()
        {
            switch (myState)
            {
                case States.COOKING:
                    ResetValues();
                    myDisplay.Clear();
                    myLight.TurnOff();
                    // Beep 3 times
                    myState = States.READY;
                    break;
            }
        }
    }
}