﻿using System;
using System.Threading;
using Microwave.Classes.Controllers;
using Microwave.Classes.Interfaces;
using NSubstitute;
using NuGet.Frameworks;
using NUnit.Framework;
using NUnit.Framework.Internal.Commands;

namespace Microwave.Test.Unit
{
    [TestFixture]
    public class CookControllerTest
    {
        private CookController uut;

        private IUserInterface ui;
        private ITimer timer;
        private IDisplay display;
        private IPowerTube powerTube;

        [SetUp]
        public void Setup()
        {
            ui = Substitute.For<IUserInterface>();
            timer = Substitute.For<ITimer>();
            display = Substitute.For<IDisplay>();
            powerTube = Substitute.For<IPowerTube>();
            powerTube.PowerTubeSize.Returns(700);
            uut = new CookController(timer, display, powerTube, ui);
        }

        [Test]
        public void StartCooking_ValidParameters_TimerStarted()
        {
            uut.StartCooking(50, 60);

            timer.Received().Start(60);
        }

        [Test]
        public void StartCooking_ValidParameters_PowerTubeStarted()
        {
            uut.StartCooking(50, 60);

            powerTube.Received().TurnOn(50);
        }

        [Test]
        public void Cooking_TimerTick_DisplayCalled()
        {
            uut.StartCooking(50, 60);

            timer.TimeRemaining.Returns(115);
            timer.TimerTick += Raise.EventWith(this, EventArgs.Empty);

            display.Received().ShowTime(1, 55);
        }

        [Test]
        public void Cooking_TimerExpired_PowerTubeOff()
        {
            uut.StartCooking(50, 60);

            timer.Expired += Raise.EventWith(this, EventArgs.Empty);

            powerTube.Received().TurnOff();
        }

        [Test]
        public void Cooking_TimerExpired_UICalled()
        {
            uut.StartCooking(50, 60);

            timer.Expired += Raise.EventWith(this, EventArgs.Empty);

            ui.Received().CookingIsDone();
        }

        [Test]
        public void Cooking_Stop_PowerTubeOff()
        {
            uut.StartCooking(50, 60);
            uut.Stop();

            powerTube.Received().TurnOff();
        }

        [Test]
        public void Cooking_Defrost_Enabled()
        {
            uut.Defrost(700);
            powerTube.Received(700);
        }

        [Test]
        public void Cooking_Defrost_ToggleTest()
        {
            uut.Defrost(700);
            uut.StartCooking(700, 30);
            uut.Stop();

            powerTube.Received(1).TurnOn(700);
            powerTube.Received(1).TurnOff();


        }
        
        [Test]    
        public void AddCookingTime_IsCooking_AddTimeCalled()
        {
            uut.StartCooking(50, 60);
            uut.AddCookingTime();

            timer.Received().AddTime(10);
        }

        [Test]
        public void Get_Max_Power()
        {
	        powerTube.PowerTubeSize.Returns(700);

	        int test = uut.MaxPower;
            Assert.AreEqual(700,test);
        }

	}
}